(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'babel-runtime/core-js/object/keys', 'path', 'fs'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('babel-runtime/core-js/object/keys'), require('path'), require('fs'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.keys, global.path, global.fs);
    global.CanIUseProvider = mod.exports;
  }
})(this, function (exports, _keys, _path, _fs) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.supportedTargets = undefined;

  var _keys2 = _interopRequireDefault(_keys);

  var _path2 = _interopRequireDefault(_path);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  var supportedTargets = exports.supportedTargets = ['chrome', 'firefox', 'opera', 'safari', 'android', 'ie', 'edge', 'ios_saf', 'op_mini', 'android', 'bb', 'op_mob', 'and_chr', 'and_ff', 'ie_mob', 'and_uc', 'samsung'];

  function isValid(node, eslintNode, targets) {
    // Filter non-matching objects and properties
    switch (eslintNode.type) {
      case 'NewExpression':
        if (!eslintNode.callee) return true;
        if (eslintNode.callee.name !== node.object) return true;
        break;
      case 'MemberExpression':
        // Pass tests if non-matching object or property
        if (!eslintNode.object || !eslintNode.property) return true;
        if (eslintNode.object.name !== node.object) return true;

        // If the property is missing from the rule, it means that only the
        // object is required to determine compatability
        if (!node.property) break;

        if (eslintNode.property.name !== node.property) return true;
        break;
      default:
        return true;
    }

    // Check the CanIUse database to see if targets are supported
    var caniuseRecord = JSON.parse((0, _fs.readFileSync)(_path2.default.join(__dirname, './caniuse/features-json/' + node.id + '.json')).toString()).stats;

    // Check if targets are supported. By default, get the latest version of each
    // target environment
    return targets.every(function (target) {
      var sortedVersions = (0, _keys2.default)(caniuseRecord[target]).sort(function (a, b) {
        return a - b;
      });

      var latestVersion = sortedVersions[sortedVersions.length - 1];
      var latest = caniuseRecord[target][latestVersion];

      return latest !== 'n';
    });
  }

  //
  // TODO: Refactor to separate module
  //

  var CanIUseProvider = [
  // ex. new ServiceWorker()
  {
    id: 'serviceworkers',
    ASTNodeType: 'NewExpression',
    object: 'ServiceWorker',
    isValid: isValid
  }, {
    id: 'serviceworkers',
    ASTNodeType: 'MemberExpression',
    object: 'navigator',
    property: 'serviceWorker',
    isValid: isValid
  },
  // ex. document.querySelector()
  {
    id: 'queryselector',
    ASTNodeType: 'MemberExpression',
    object: 'document',
    property: 'querySelector',
    isValid: isValid
  },
  // ex. WebAssembly
  {
    id: 'wasm',
    ASTNodeType: 'MemberExpression',
    object: 'WebAssembly',
    isValid: isValid
  },
  // ex. IntersectionObserver
  {
    id: 'intersectionobserver',
    ASTNodeType: 'NewExpression',
    object: 'IntersectionObserver',
    isValid: isValid
  },
  // ex. document.currentScript()
  {
    id: 'document-currentscript',
    ASTNodeType: 'MemberExpression',
    object: 'document',
    property: 'currentScript',
    isValid: isValid
  }];

  exports.default = CanIUseProvider;
});