(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['module', 'exports', '../Lint'], factory);
  } else if (typeof exports !== "undefined") {
    factory(module, exports, require('../Lint'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod, mod.exports, global.Lint);
    global.compat = mod.exports;
  }
})(this, function (module, exports, _Lint) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _Lint2 = _interopRequireDefault(_Lint);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  exports.default = {
    meta: {
      docs: {
        description: 'Ensure cross-browser API compatability',
        category: 'Compatability',
        recommended: true
      },
      fixable: 'code',
      schema: []
    },
    create: function create(context) {
      function lint(node) {
        var isValid = (0, _Lint2.default)(node, context.settings.targets);

        // HACK: Eventually, we'll have an error message returned from Lint
        // const { isValid, message } = Lint(context);

        if (!isValid) {
          context.report({
            node: node,
            message: 'Unsupported API being used'
          });
        }
      }

      return {
        // HACK: Ideally, rules will be generated at runtime. Each rule will have
        //       have the ability to register itself to run on specific AST
        //       nodes. For now, we're using the `CallExpression` node since
        //       its what most rules will run on
        MemberExpression: lint,
        NewExpression: lint
      };
    }
  };
  module.exports = exports['default'];
});